<?php

namespace OzzModz\WebsiteVerification\Admin\Controller;

use XF\Mvc\ParameterBag;

class Website extends \XF\Admin\Controller\AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('ozzmodz_wv_manage');
	}

	public function actionIndex(ParameterBag $params)
	{
		if ($params->website_id)
		{
			$website = $this->assertWebsiteExists($params->website_id);

			$viewParams = [
				'website' => $website
			];

			return $this->view(
				'OzzModz\WebsiteVerification:Website\View',
				'ozzmodz_wv_website_view',
				$viewParams
			);
		}
		else
		{
			$page = $this->filterPage();
			$perPage = 20;

			/** @var \OzzModz\WebsiteVerification\Repository\Website $websiteRepo */
			$websiteRepo = $this->repository('OzzModz\WebsiteVerification:Website');

			$logFinder = $websiteRepo->findWebsitesForList()
				->limitByPage($page, $perPage);

			$filters = $this->getWebsiteFilterInput(true);
			$this->applyWebsiteFilters($logFinder);

			$viewParams = [
				'websites' => $logFinder->fetch(),
				'filters' => $filters,
				'page' => $page,
				'perPage' => $perPage,
				'total' => $logFinder->total(),
			];

			return $this->view(
				'OzzModz\WebsiteVerification:Website\List',
				'ozzmodz_wv_website_list',
				$viewParams
			);
		}
	}

	public function actionRefineSearch()
	{
		$viewParams = [
			'datePresets' => \XF::language()->getDatePresets(),
			'conditions' => $this->getWebsiteFilterInput(),
		];

		return $this->view(
			'OzzModz\WebsiteVerification:Website\RefineSearch',
			'ozzmodz_wv_website_refine_search',
			$viewParams
		);
	}

	protected function applyWebsiteFilters(\XF\Mvc\Entity\Finder $finder)
	{
		$filters = $this->getWebsiteFilterInput();

		if ($filters['domain'])
		{
			$finder->where('domain', 'LIKE', $finder->escapeLike($filters['domain'], '%?%'));
		}

		if ($filters['username'])
		{
			$finder->where('User.username', 'LIKE', $finder->escapeLike($filters['username'], '%?%'));
		}

		if ($filters['start'])
		{
			$finder->where('date_added', '>', $filters['start']);
		}
		if ($filters['end'])
		{
			$finder->where('date_added', '<', $filters['end']);
		}
	}

	protected function getWebsiteFilterInput($removeEmpty = false): array
	{
		$input = $this->filter([
			'domain' => 'str',
			'username' => 'str',
			'start' => 'datetime',
			'end' => 'datetime',
		]);

		if ($removeEmpty)
		{
			$input = array_filter($input);
		}

		return $input;
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\Error|\XF\Mvc\Reply\Redirect|\XF\Mvc\Reply\View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionDelete(ParameterBag $params)
	{
		$website = $this->assertWebsiteExists($params->website_id);

		/** @var \XF\ControllerPlugin\Delete $plugin */
		$plugin = $this->plugin('XF:Delete');

		return $plugin->actionDelete(
			$website,
			$this->buildLink('website-verification/websites/delete', $website),
			$this->buildLink('website-verification/websites/edit', $website),
			$this->buildLink('website-verification/websites'),
			$website->domain
		);
	}

	/**
	 * @param $id
	 * @param null $with
	 * @param null $phrase
	 * @return \XF\Mvc\Entity\Entity|\OzzModz\WebsiteVerification\Entity\Website
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertWebsiteExists($id, $with = null, $phrase = null)
	{
		return $this->assertRecordExists('OzzModz\WebsiteVerification:Website', $id, $with, $phrase);
	}
}