<?php

namespace OzzModz\WebsiteVerification\Api\Controller;

use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

class Website extends \XF\Api\Controller\AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertApiScopeByRequestMethod('ozzmodz_wv_website', ['delete' => 'delete']);
	}

	/**
	 * @api-desc Gets information about the specified website.
	 *
	 * @api-out OzzModz_WebsiteVerification_Website $website
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionGet(ParameterBag $params)
	{
		$website = $this->assertWebsiteExists($params->website_id);

		$result = [
			'website' => $website->toApiResult(Entity::VERBOSITY_VERBOSE)
		];
		return $this->apiResult($result);
	}

	/**
	 * @api-desc Deletes the specified website.
	 *
	 * @api-out true $success
	 *
	 * @param ParameterBag $params
	 * @return \XF\Api\Mvc\Reply\ApiResult|\XF\Mvc\Reply\AbstractReply|\XF\Mvc\Reply\Error
	 * @throws \XF\Mvc\Reply\Exception
	 * @throws \XF\PrintableException
	 */
	public function actionDelete(ParameterBag $params)
	{
		$this->assertAdminPermission('ozzmodz_wv_manage');

		$website = $this->assertWebsiteExists($params->website_id);
		if (!$website->preDelete())
		{
			return $this->error($website->getErrors());
		}

		$website->delete();

		return $this->apiSuccess();
	}

	/**
	 * @param $websiteId
	 * @param null $with
	 * @return \XF\Mvc\Entity\Entity|\OzzModz\WebsiteVerification\Entity\Website
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertWebsiteExists($websiteId, $with = null)
	{
		return $this->assertRecordExists('OzzModz\WebsiteVerification:Website', $websiteId, $with, 'ozzmodz_vw_requested_website_not_found');
	}
}
