<?php

namespace OzzModz\WebsiteVerification\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property string $validation_token
 * @property string $customer_token
 * @property string $license_token
 * @property string $subscription_token
 * @property int $is_valid
 * @property int $can_transfer
 * @property int $domain_match
 * @property string $domain
 * @property int $last_check_date
 *
 * RELATIONS
 * @property Website $Website
 */
class XenforoLicense extends Entity
{
	public function setFromApiResponse($licenseInfo)
	{
		$this->bulkSet([
			'validation_token' => $licenseInfo['validation_token'],
			'customer_token' => $licenseInfo['customer_token'] ?? null,
			'license_token' => $licenseInfo['license_token'] ?? null,
			'subscription_token' => $licenseInfo['subscription_token'] ?? null,
			'is_valid' => $licenseInfo['is_valid'] ?? null,
			'can_transfer' => $licenseInfo['can_transfer'] ?? null,
			'domain_match' => $licenseInfo['domain_match'] ?? null,
			'domain' => $licenseInfo['domain'] ?? null
		]);
	}

	protected function setupApiResultData(\XF\Api\Result\EntityResult $result, $verbosity = self::VERBOSITY_NORMAL, array $options = [])
	{
	}

	protected function _postSave()
	{
		if ($this->isInsert())
		{
			$licenseRepo = $this->getXenforoLicenseRepo();
			$licenseRepo->runLicenseCheckJobIfNeeded();
		}

		if ($this->isInsert() || $this->isChanged('is_valid'))
		{
			$user = $this->Website && $this->Website->User ? $this->Website->User : null;
			if ($user)
			{
				$user->rebuildOzzModzWebsiteVerificationCounters();
				$user->save();
			}
		}
	}

	protected function _postDelete()
	{
		$user = $this->Website && $this->Website->User ? $this->Website->User : null;
		if ($user)
		{
			$user->rebuildOzzModzWebsiteVerificationCounters();
			$user->save();
		}
	}

	/**
	 * @param Structure $structure
	 * @return Structure
	 */
	public static function getStructure(Structure $structure)
	{
		$structure->table = 'xf_ozzmodz_wv_xenforo_license';
		$structure->shortName = 'OzzModz\WebsiteVerification:XenforoLicense';
		$structure->primaryKey = 'validation_token';
		$structure->columns = [
			'validation_token' => ['type' => static::STR, 'maxLength' => 50, 'required' => true, 'api' => true],
			'customer_token' => ['type' => static::STR, 'maxLength' => 50, 'nullable' => true, 'required' => true, 'api' => true],
			'license_token' => ['type' => static::STR, 'maxLength' => 50, 'nullable' => true, 'required' => true, 'api' => true],
			'subscription_token' => ['type' => static::STR, 'maxLength' => 50, 'nullable' => true, 'required' => true, 'api' => true],
			'is_valid' => ['type' => static::UINT, 'maxLength' => 255, 'nullable' => true, 'default' => '1', 'api' => true],
			'can_transfer' => ['type' => static::INT, 'maxLength' => 128, 'nullable' => true, 'required' => true, 'api' => true],
			'domain_match' => ['type' => static::INT, 'maxLength' => 128, 'nullable' => true, 'required' => true, 'api' => true],
			'domain' => ['type' => static::STR, 'maxLength' => 255, 'nullable' => true, 'required' => true, 'api' => true],
			'last_check_date' => ['type' => static::INT, 'default' => \XF::$time, 'api' => true],
		];

		$structure->relations = [
			'Website' => [
				'entity' => 'OzzModz\WebsiteVerification:Website',
				'type' => self::TO_ONE,
				'conditions' => [
					['xenforo_validation_token', '=', '$validation_token']
				],
				'primary' => true
			]
		];

		return $structure;
	}

	/**
	 * @return \XF\Mvc\Entity\Repository|\OzzModz\WebsiteVerification\Repository\XenforoLicense
	 */
	public function getXenforoLicenseRepo()
	{
		return $this->repository('OzzModz\WebsiteVerification:XenforoLicense');
	}
}