<?php

namespace OzzModz\WebsiteVerification\Job;


class WebsiteCheck extends \XF\Job\AbstractJob
{
	public function run($maxRunTime)
	{
		$startTime = microtime(true);

		/** @var \OzzModz\WebsiteVerification\Repository\Website $repository */
		$repository = \XF::repository('OzzModz\WebsiteVerification:Website');
		$websites = $repository->findWebsitesToCheck()->fetch();
		if (!$websites->count())
		{
			$nextCheckTime = $repository->getNextWebsiteCheckTime();
			if ($nextCheckTime)
			{
				$resume = $this->resume();
				$resume->continueDate = $nextCheckTime;

				return $resume;
			}
			else
			{
				return $this->complete();
			}
		}

		/** @var \OzzModz\WebsiteVerification\Entity\Website $website */
		foreach ($websites as $website)
		{
			$filePath = $website->getVerificationFilePath();

			/** @var \OzzModz\WebsiteVerification\Helper\Http $httpHelper */
			$httpHelper = $this->app->helper('OzzModz\WebsiteVerification:Http');
			$contents = $httpHelper->getFileContents('http://' . $filePath, $error)
				|| $httpHelper->getFileContents('https://' . $filePath, $error);

			if ($contents == $website->verification_code)
			{
				$website->file_verified = true;
			}
			else
			{
				$website->file_verified = false;
			}

			/** @var \OzzModz\WebsiteVerification\Helper\Dns $dnsHelper */
			$dnsHelper = $this->app->helper('OzzModz\WebsiteVerification:Dns');
			$found = $dnsHelper->findTxtRecord($website->domain, $website->getTxtRecord());
			if ($found)
			{
				$website->dns_verified = true;
			}
			else
			{
				$website->dns_verified = false;
			}

			$website->validation_date = \XF::$time;
			$website->save();

			if (microtime(true) - $startTime >= $maxRunTime)
			{
				break;
			}
		}

		return $this->resume();
	}

	public function getStatusMessage()
	{
		return \XF::phrase('ozzmodz_wv_checking_websites...');
	}

	public function canCancel()
	{
		return false;
	}

	public function canTriggerByChoice()
	{
		return true;
	}
}