<?php

namespace OzzModz\WebsiteVerification\Job;


class XenforoLicenseCheck extends \XF\Job\AbstractJob
{
	public function run($maxRunTime)
	{
		$startTime = microtime(true);

		/** @var \OzzModz\WebsiteVerification\Repository\XenforoLicense $repository */
		$repository = \XF::repository('OzzModz\WebsiteVerification:XenforoLicense');
		$licenses = $repository->findLicensesToCheck()->fetch();
		if (!$licenses->count())
		{
			$nextCheckTime = $repository->getNextLicenseCheckTime();
			if ($nextCheckTime)
			{
				$resume = $this->resume();
				$resume->continueDate = $nextCheckTime;

				return $resume;
			}
			else
			{
				return $this->complete();
			}
		}

		/** @var \OzzModz\WebsiteVerification\Entity\XenforoLicense $license */
		foreach ($licenses AS $license)
		{
			/** @var \OzzModz\WebsiteVerification\Helper\XenforoLicenseApi $licenseHelper */
			$licenseHelper = $this->app->helper('OzzModz\WebsiteVerification:XenforoLicenseApi');
			$licenseInfo = $licenseHelper->getLicenseInfo($license->validation_token, $license->Website ? $license->Website->domain : null);

			if ($licenseHelper->getError())
			{
				if ($licenseHelper->getStatusCode() == 404)
				{
					$license->is_valid = false;
				}
				else
				{
					$resume = $this->resume();
					$resume->continueDate = \XF::$time + 3600;
					return $resume;
				}
			}
			else
			{
				$license->setFromApiResponse($licenseInfo);
			}

			$license->last_check_date = \XF::$time;
			$license->save();

			if (microtime(true) - $startTime >= $maxRunTime)
			{
				break;
			}
		}

		return $this->resume();
	}

	public function getStatusMessage()
	{
		return \XF::phrase('ozzmodz_wv_checking_xf_licenses...');
	}

	public function canCancel()
	{
		return false;
	}

	public function canTriggerByChoice()
	{
		return true;
	}
}