<?php

namespace OzzModz\WebsiteVerification;

use XF\Mvc\Entity\Entity;

/**
 * Class Listener
 *
 * @package OzzModz\WebsiteVerification
 */
class Listener
{
	/**
	 * Allows direct modification of the Entity structure.
	 *
	 * Event hint: Fully qualified name of the root class that was called.
	 *
	 * @param \XF\Mvc\Entity\Manager $em Entity Manager object.
	 * @param \XF\Mvc\Entity\Structure $structure Entity Structure object.
	 */
	public static function userEntityStructure(\XF\Mvc\Entity\Manager $em, \XF\Mvc\Entity\Structure &$structure)
	{
		$structure->columns['ozzmodz_wv_website_count'] = ['type' => Entity::UINT, 'default' => 0, 'api' => true];
		$structure->columns['ozzmodz_wv_website_verified_count'] = ['type' => Entity::UINT, 'default' => 0, 'api' => true];
	}

	/**
	 * Event fires after the Entity specific _postDelete() method is called and after the post delete
	 * of any behaviors.
	 *
	 * Event hint: Fully qualified name of the root class that was called.
	 *
	 * @param Entity $entity Entity object.
	 */
	public static function userEntityPostDelete(Entity $entity)
	{
		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $entity */
		$entity->db()->delete('xf_ozzmodz_wv_website', 'user_id = ?', $entity->user_id);
	}

	/**
	 * Called while testing a user against user criteria in \XF\Criteria\User::isMatch() for trophies,
	 * notices etc.
	 *
	 * @param string $rule Text identifying the criteria that should be
	 *                                     checked.
	 * @param array $data Data defining the conditions of the criteria.
	 * @param \XF\Entity\User $user User entity object to be used in the criteria
	 *                                     checks.
	 * @param bool $returnValue The event code should set this to true if a criteria
	 *                                     check matches.
	 */
	public static function criteriaUser(string $rule, array $data, \XF\Entity\User $user, bool &$returnValue)
	{
		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $user */
		switch ($rule)
		{
			case 'ozzmodz_wv_website_count_min':
				if (isset($user->ozzmodz_wv_website_count) && $user->ozzmodz_wv_website_count <= $data['websites'])
				{
					$returnValue = true;
				}
				break;
			case 'ozzmodz_wv_website_count_max':
				if (isset($user->ozzmodz_wv_website_count) && $user->ozzmodz_wv_website_count >= $data['websites'])
				{
					$returnValue = true;
				}

				break;
			case 'ozzmodz_wv_website_verified_count_min':
				if (isset($user->ozzmodz_wv_website_verified_count) && $user->ozzmodz_wv_website_verified_count <= $data['websites'])
				{
					$returnValue = true;
				}
				break;
			case 'ozzmodz_wv_website_verified_count_max':
				if (isset($user->ozzmodz_wv_website_verified_count) && $user->ozzmodz_wv_website_verified_count >= $data['websites'])
				{
					$returnValue = true;
				}
				break;
		}
	}

}