<?php

namespace OzzModz\WebsiteVerification\Pub\Controller;

use XF\Mvc\ParameterBag;

class Website extends \XF\Pub\Controller\AbstractController
{
	/**
	 * @param $action
	 * @param ParameterBag $params
	 * @return void
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function preDispatchController($action, ParameterBag $params)
	{
		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		if (!$visitor->hasPermission('ozzmodz_wv', 'viewOwn'))
		{
			throw $this->exception($this->noPermission());
		}
	}

	public function actionIndex()
	{
		return $this->redirect('account/website-verification');
	}

	public function actionAdd()
	{
		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		if (!$visitor->canAddWebsiteVerificationDomains($error))
		{
			return $this->noPermission();
		}

		if ($this->isPost())
		{
			$creator = $this->setupWebsiteCreate();
			if (!$creator->validate($errors))
			{
				return $this->error($errors);
			}

			$creator->save();
			$this->finalizeWebsiteCreate($creator);

			return $this->redirect($this->buildLink('account/website-verification'));
		}
		/** @var \OzzModz\WebsiteVerification\Entity\Website $domain */
		$domain = $this->em()->create('OzzModz\WebsiteVerification:Website');

		$viewParams = [
			'domain' => $domain,
		];
		return $this->view('OzzModz\WebsiteVerification:Website\Add', 'ozzmodz_wv_website_add', $viewParams);
	}

	protected function setupWebsiteCreate()
	{
		/** @var \OzzModz\WebsiteVerification\Service\Website\Create $creator */
		$creator = $this->service('OzzModz\WebsiteVerification:Website\Create');
		$creator->setDomain($this->filter('domain', 'str'));

		return $creator;
	}

	protected function finalizeWebsiteCreate(\OzzModz\WebsiteVerification\Service\Website\Create $creator)
	{
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\Error|\XF\Mvc\Reply\Redirect|\XF\Mvc\Reply\View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionVerify(ParameterBag $params)
	{
		$website = $this->assertViewableWebsite($params->website_id);
		if ($this->isPost())
		{
			$this->assertNotFlooding('ozzmodz_wv_website');

			/** @var \OzzModz\WebsiteVerification\Service\Website\Verify $verifier */
			$verifier = $this->service('OzzModz\WebsiteVerification:Website\Verify', $website);

			$validationToken = $this->filter('xenforo_validation_token', 'str');
			if ($validationToken)
			{
				$verifier->setXenforoValidationToken($validationToken);
			}

			if (!$verifier->validate($errors))
			{
				return $this->error($errors);
			}

			$verifier->save();

			return $this->redirect($this->buildLink('account/website-verification'));
		}

		$viewParams = [
			'website' => $website,
		];
		return $this->view('OzzModz\WebsiteVerification:Website\Verify', 'ozzmodz_wv_website_verify', $viewParams);
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionDownload(ParameterBag $params)
	{
		$website = $this->assertViewableWebsite($params->website_id);

		$this->setResponseType('raw');
		return $this->view('OzzModz\WebsiteVerification:Website\Download', '', ['website' => $website]);
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\Error|\XF\Mvc\Reply\Redirect|\XF\Mvc\Reply\View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionDelete(ParameterBag $params)
	{
		$website = $this->assertViewableWebsite($params->website_id);
		if (!$website->canDelete($error))
		{
			return $this->error($error);
		}

		/** @var \XF\ControllerPlugin\Delete $deletePlugin */
		$deletePlugin = $this->plugin('XF:Delete');
		return $deletePlugin->actionDelete(
			$website,
			$this->buildLink('website-verification/delete', $website),
			$this->buildLink('account/website-verification'),
			$this->buildLink('account/website-verification'),
			$website->domain
		);
	}

	/**
	 * @param $id
	 * @param $with
	 * @param $phraseKey
	 * @return \OzzModz\WebsiteVerification\Entity\Website
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertViewableWebsite($id, $with = null, $phraseKey = null)
	{
		/** @var \OzzModz\WebsiteVerification\Entity\Website $website */
		$website = $this->assertRecordExists('OzzModz\WebsiteVerification:Website', $id, $with, $phraseKey);
		if (!$website->canView($error))
		{
			throw $this->exception($this->noPermission($error));
		}

		return $website;
	}
}