<?php

namespace OzzModz\WebsiteVerification\Repository;

use XF\Mvc\Entity\ArrayCollection;

class Website extends \XF\Mvc\Entity\Repository
{
	/**
	 * @return \XF\Mvc\Entity\Finder|\OzzModz\WebsiteVerification\Finder\Website
	 */
	public function findWebsitesForList()
	{
		return $this->finder('OzzModz\WebsiteVerification:Website')
			->setDefaultOrder('date_added', 'DESC');
	}

	/**
	 * @return \XF\Mvc\Entity\Finder|\OzzModz\WebsiteVerification\Finder\Website
	 */
	public function findWebsitesForUser(\XF\Entity\User $user)
	{
		return $this->findWebsitesForList()
			->with('XenforoLicense')
			->byUser($user);
	}

	public function getVerifiedWebsitesForUser(\XF\Entity\User $user)
	{
		if (!$user->user_id)
		{
			return new ArrayCollection([]);
		}

		$websites = $this->findWebsitesForList()
			->with('XenforoLicense')
			->byUser($user)
			->fetch();

		return $websites->filter(function (\OzzModz\WebsiteVerification\Entity\Website $website) {
			return $website->isVerified();
		});
	}

	public function findWebsitesToCheck()
	{
		$licenseFinder = $this->finder('OzzModz\WebsiteVerification:Website');

		$checkInterval = $this->options()->ozzmodz_wv_xfLicenseCheckInterval * 86400;
		if (!$checkInterval)
		{
			$licenseFinder->whereImpossible();
			return $licenseFinder;
		}

		return $licenseFinder->where('last_check_date', '<', \XF::$time - $checkInterval)
			->setDefaultOrder('last_check_date');
	}

	public function getNextWebsiteCheckTime()
	{
		$checkInterval = $this->options()->ozzmodz_wv_xfLicenseCheckInterval * 86400;
		if (!$checkInterval)
		{
			return null;
		}

		$nextCheckDate = $this->db()->fetchOne("
			SELECT MIN(validation_date) 
			FROM xf_ozzmodz_wv_website
			WHERE validation_date < ?
		", \XF::$time - $checkInterval);

		return $nextCheckDate ?: null;
	}

	public function runWebsiteCheckJobIfNeeded()
	{
		$nextLicenseCheckTime = $this->getNextWebsiteCheckTime();
		if ($nextLicenseCheckTime)
		{
			$this->app()->jobManager()->enqueueLater(
				'ozzmodzWvWebsiteChecker',
				$nextLicenseCheckTime,
				'OzzModz\WebsiteVerification:WebsiteCheck'
			);
		}
	}

	/**
	 * @return string
	 */
	public function generateVerificationCode()
	{
		$finder = $this->finder('OzzModz\WebsiteVerification:Website');

		do
		{
			$verificationCode = \XF::generateRandomString(32);

			$found = $finder->resetWhere()
				->where('verification_code', $verificationCode)
				->fetchOne();

			if (!$found)
			{
				break;
			}
		} while (true);

		return $verificationCode;
	}
}