<?php

namespace OzzModz\WebsiteVerification\Service\Website;

use XF\Service\ValidateAndSavableTrait;

class Create extends \XF\Service\AbstractService
{
	use ValidateAndSavableTrait;

	/**
	 * @var \OzzModz\WebsiteVerification\Entity\Website
	 */
	protected $website;

	protected $performValidations = true;

	public function __construct(\XF\App $app)
	{
		parent::__construct($app);

		$this->website = $this->app->em()->create('OzzModz\WebsiteVerification:Website');
		$this->setupDefaults();
	}

	protected function setupDefaults()
	{
		/** @var \OzzModz\WebsiteVerification\Repository\Website $websiteRepo */
		$websiteRepo = $this->repository('OzzModz\WebsiteVerification:Website');
		$verificationCode = $websiteRepo->generateVerificationCode();

		$this->setVerificationCode($verificationCode);
		$this->setUser(\XF::visitor());
	}

	public function setDomain($domain)
	{
		$this->website->domain = $domain;
	}

	public function setUser(\XF\Entity\User $user)
	{
		$this->website->user_id = $user->user_id;
	}

	public function setVerificationCode($verificationCode)
	{
		$this->website->verification_code = $verificationCode;
	}

	public function setPerformValidations($perform)
	{
		$this->performValidations = (bool) $perform;
	}

	public function getPerformValidations()
	{
		return $this->performValidations;
	}

	protected function finalSetup()
	{
	}

	protected function _validate()
	{
		$errors = [];
		$website = $this->website;

		if ($this->performValidations)
		{
			$options = $this->app->options();

			$bannedDomains = \XF\Util\Arr::stringToArray($options->ozzmodz_wv_domainBlacklist, '/\r?\n/');
			if (in_array($website->domain, $bannedDomains))
			{
				$errors[] = \XF::phrase('ozzmodz_wv_this_domain_is_banned_by_administrator');
				return $errors;
			}

			if ($options->ozzmodz_wv_existingDomains && gethostbyname($website->domain) === $website->domain)
			{
				$errors[] = \XF::phrase('ozzmodz_wv_failed_to_resolve_hostname');
			}

			$existingUserWebsites = $this->finder('OzzModz\WebsiteVerification:Website')
				->where([
					['domain', '=', $website->domain],
					['user_id', '=', $website->user_id],
				])
				->total();

			if ($existingUserWebsites)
			{
				$errors[] = \XF::phrase('ozzmodz_wv_you_already_have_added_this_domain');
			}
			else
			{
				$existingVerifiedWebsites = $this->finder('OzzModz\WebsiteVerification:Website')
					->where('domain', '=', $website->domain)
					->total();

				if ($existingVerifiedWebsites)
				{
					$errors[] = \XF::phrase('ozzmodz_wv_this_domain_is_already_verified_by_another_user');
				}
			}
		}

		$this->finalSetup();
		$website->preSave();
		$errors += $website->getErrors();

		if (!$website->hasChanges())
		{
			$errors = 'nothing verified';
		}

		return $errors;
	}

	protected function _save()
	{
		$website = $this->website;
		$website->save();

		return $website;
	}
}