<?php

namespace OzzModz\WebsiteVerification;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	// ################################## INSTALL ###########################################

	public function installStep1()
	{
		$sm = $this->schemaManager();

		foreach ($this->getTables() as $tableName => $callback)
		{
			$sm->createTable($tableName, $callback);
		}
	}

	public function installStep2()
	{
		$sm = $this->schemaManager();
		foreach ($this->getAlters() as $table => $schema)
		{
			if ($sm->tableExists($table))
			{
				$sm->alterTable($table, $schema);
			}
		}
	}

	// ################################## UNINSTALL ###########################################

	public function uninstallStep1()
	{
		$sm = $this->schemaManager();

		foreach (array_keys($this->getTables()) as $tableName)
		{
			$sm->dropTable($tableName);
		}
	}

	public function uninstallStep2()
	{
		$sm = $this->schemaManager();
		foreach ($this->getReverseAlters() as $table => $schema)
		{
			if ($sm->tableExists($table))
			{
				$sm->alterTable($table, $schema);
			}
		}
	}

	public function postUpgrade($previousVersion, array &$stateChanges)
	{
		if ($previousVersion)
		{
			if ($previousVersion < 2000102)
			{
				$this->app->jobManager()->enqueueUnique(
					'ozzmodz_wv_user',
					'OzzModz\WebsiteVerification:UserWebsitesRebuild',
					[],
					false
				);
			}
		}
	}

	// ################################## UPGRADE ###########################################

	public function upgrade2000102Step1()
	{
		$this->schemaManager()->alterTable('xf_user', function (Alter $table) {
			$table->addColumn('ozzmodz_wv_website_count', 'int')->setDefault(0);
			$table->addColumn('ozzmodz_wv_website_verified_count', 'int')->setDefault(0);
		});
	}

	// ################################## DATA ###########################################

	protected function getTables(): array
	{
		$tables = [];

		$tables['xf_ozzmodz_wv_website'] = function (Create $table) {
			$table->addColumn('website_id', 'int')->autoIncrement();
			$table->addColumn('domain', 'varchar', 255);

			$table->addColumn('user_id', 'int');
			$table->addColumn('verification_code', 'varbinary', 32);

			$table->addColumn('xenforo_validation_token', 'varchar', 50)->setDefault('');

			$table->addColumn('date_added', 'int')->setDefault(0);
			$table->addColumn('validation_date', 'int')->setDefault(0);
			$table->addColumn('last_update', 'int')->setDefault(0);

			$table->addColumn('dns_verified', 'tinyint')->setDefault(0);
			$table->addColumn('file_verified', 'tinyint')->setDefault(0);

			$table->addKey('user_id');
			$table->addKey('domain');
			$table->addKey('xenforo_validation_token');
			$table->addUniqueKey('verification_code');
		};

		$tables['xf_ozzmodz_wv_xenforo_license'] = function (Create $table) {
			$table->addColumn('validation_token', 'varchar', 50)->primaryKey();
			$table->addColumn('customer_token', 'varchar', 50);
			$table->addColumn('license_token', 'varchar', 50)->nullable();
			$table->addColumn('subscription_token', 'varchar', 50)->nullable();

			$table->addColumn('is_valid', 'tinyint')->setDefault(1);
			$table->addColumn('can_transfer', 'bool')->nullable();
			$table->addColumn('domain_match', 'bool')->nullable();
			$table->addColumn('domain', 'varchar', 255)->nullable();

			$table->addColumn('last_check_date', 'int')->setDefault(0);
		};

		return $tables;
	}

	/**
	 * @return array
	 */
	protected function getAlters()
	{
		$alters = [];

		$alters['xf_user'] = function (Alter $table) {
			$table->addColumn('ozzmodz_wv_website_count', 'int')->setDefault(0);
			$table->addColumn('ozzmodz_wv_website_verified_count', 'int')->setDefault(0);
		};

		return $alters;
	}

	/**
	 * @return array
	 */
	protected function getReverseAlters()
	{
		$alters = [];

		$alters['xf_user'] = function (Alter $table) {
			$table->dropColumns([
				'ozzmodz_wv_website_count',
				'ozzmodz_wv_website_verified_count'
			]);
		};

		return $alters;
	}
}