<?php

namespace OzzModz\WebsiteVerification\XF\Pub\Controller;

use XF\Mvc\ParameterBag;

class Member extends XFCP_Member
{
	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\AbstractReply|\XF\Mvc\Reply\View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionWebsites(ParameterBag $params)
	{
		/** @var \OzzModz\WebsiteVerification\XF\Entity\User $user */
		$user = $this->assertViewableUser($params->user_id);
		if (!$user->canViewWebsiteVerificationDomains($error))
		{
			return $this->noPermission($error);
		}

		$page = $this->filterPage();
		$perPage = 20;

		/** @var \OzzModz\WebsiteVerification\Repository\Website $websiteRepo */
		$websiteRepo = $this->repository('OzzModz\WebsiteVerification:Website');
		$websiteFinder = $websiteRepo->findWebsitesForUser($user);

		$websiteFinder->limitByPage($page, $perPage);

		$websites = $websiteFinder->fetch();
		$total = $websiteFinder->total();

		$viewParams = [
			'websites' => $websites,
			'total' => $total,
			'page' => $page,
			'perPage' => $perPage,
		];

		return $this->view('XF:Member\VerifiedWebsites', 'ozzmodz_wv_member_website_list', $viewParams);
	}
}