<?php

namespace OzzModz\WebsiteVerification\XFA\RMMarketplace\Pub\Controller;

use XF\Mvc\ParameterBag;


class License extends XFCP_License
{
	public function actionSetUrl(ParameterBag $params)
	{
		if ($this->options()->ozzmodz_wv_xfa_rmmp_replaceLicenseUrl)
		{
			$license = $this->assertOwnedLicense($params->license_id);

			if ($this->isPost())
			{
				/** @var \OzzModz\WebsiteVerification\Entity\Website $website */
				$website = $this->em()->find('OzzModz\WebsiteVerification:Website', $this->filter('website_id', 'uint'));
				if (!$website)
				{
					return $this->notFound();
				}

				if (!$website->isVerified())
				{
					return $this->error(\XF::phrase('ozzmodz_wv_website_is_not_verified')); // TODO
				}

				$blacklistedUrls = $this->options()->xfa_rmmp_licence_blacklist;
				if ($blacklistedUrls)
				{
					$blacklistedDomains = preg_split('/[\r\n]+/', $blacklistedUrls, -1, PREG_SPLIT_NO_EMPTY);
					if (in_array($website->domain, $blacklistedDomains))
					{
						return $this->error(\XF::phrase('xfa_rmmp_invalid_domain_provided'));
					}
				}

				$license->siteurl = $website->domain;
				$license->save();

				return $this->redirect($this->buildLink('resources/market-place-user/licenses'));
			}
			else
			{
				/** @var \OzzModz\WebsiteVerification\Repository\Website $websiteRepo */
				$websiteRepo = $this->repository('OzzModz\WebsiteVerification:Website');
				$websites = $websiteRepo->getVerifiedWebsitesForUser(\XF::visitor());
				if (!$websites->count())
				{
					return $this->error(\XF::phrase('ozzmodz_wv_you_dont_have_any_verified_website_verify_x_link', [
						'link' => $this->app()->router('public')->buildLink('account/website-verification')
					]));
				}

				$viewParams = [
					'license' => $license,
					'websites' => $websites
				];

				return $this->view('', 'xfa_rmmp_license_url_set_form', $viewParams);
			}
		}

		return parent::actionSetUrl($params);
	}

	protected function assertRenewableLicense($licenseId, array $extraWith = [])
	{
		/** @var \OzzModz\WebsiteVerification\Repository\Website $websiteRepo */
		$websiteRepo = $this->repository('OzzModz\WebsiteVerification:Website');
		$hasVerifiedWebsites = $websiteRepo->getVerifiedWebsitesForUser(\XF::visitor())->count();
		if (!$hasVerifiedWebsites)
		{
			throw $this->errorException(\XF::phrase('ozzmodz_wv_you_dont_have_any_verified_website_verify_x_link', [
				'link' => $this->app()->router('public')->buildLink('account/website-verification')
			]));
		}

		return parent::assertRenewableLicense($licenseId, $extraWith);
	}

	protected function assertOwnedLicense($licenseId, array $extraWith = [])
	{
		/** @var \OzzModz\WebsiteVerification\Repository\Website $websiteRepo */
		$websiteRepo = $this->repository('OzzModz\WebsiteVerification:Website');
		$hasVerifiedWebsites = $websiteRepo->getVerifiedWebsitesForUser(\XF::visitor())->count();
		if (!$hasVerifiedWebsites)
		{
			throw $this->errorException(\XF::phrase('ozzmodz_wv_you_dont_have_any_verified_website_verify_x_link', [
				'link' => $this->app()->router('public')->buildLink('account/website-verification')
			]));
		}

		return parent::assertOwnedLicense($licenseId, $extraWith);
	}
}